package de.gurkengewuerz.ripmewrapper.handler;

import de.gurkengewuerz.ripmewrapper.Webserver;
import org.apache.commons.io.FilenameUtils;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.AbstractHandler;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by gurkengewuerz.de on 09.10.2018.
 */
public class StaticHandler extends AbstractHandler {

    private String get403() throws IOException {
        File f = new File(Webserver.getStaticdir(), "403.html");
        if (f.exists() && f.canRead()) return getFile(f, false);
        return "<h1>403 Forbidden</h1><br/><img src='https://http.cat/403' alt='403 forbidden'/>";
    }

    private String get404() throws IOException {
        File f = new File(Webserver.getStaticdir(), "404.html");
        if (f.exists() && f.canRead()) return getFile(f, false);
        return "<h1>404 Not Found</h1><br/><img src='https://http.cat/404' alt='404 not found'/>";
    }

    private byte[] getBytes(File f) throws IOException {
        return Files.readAllBytes(f.toPath());
    }

    private String getContent(File f) throws IOException {
        return new String(getBytes(f), "utf-8");
    }

    private String getFile(File f, boolean get) throws IOException {
        String data = getContent(f);
        if (get) {
            if (data.startsWith("<protected/>")) {
                return get403();
            }
        }
        data = data.replace("<protected/>", "");

        Pattern pattern = Pattern.compile("<contain file=\"(.*?)\"/>");
        Matcher matcher = pattern.matcher(data);
        List<String> matches = new ArrayList<>();
        while (matcher.find()) {
            matches.add(matcher.group());
        }
        for (String match : matches) {
            Pattern pPattern = Pattern.compile("<contain\\sfile=\"([^\"]+)");
            Matcher pMatcher = pPattern.matcher(match);
            if (pMatcher.find()) {
                File fContain = new File(Webserver.getStaticdir(), pMatcher.group(1));
                if (fContain.exists() && fContain.isFile() && fContain.canRead()) {
                    data = data.replace(match, getFile(fContain, false));
                } else {
                    Logger.getLogger(StaticHandler.class.getName()).log(Level.SEVERE, "File not exists " + pMatcher.group(1));
                }
            }
        }
        return data;
    }

    @Override
    public void handle(String s, Request request, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws IOException, ServletException {
        Logger.getLogger(StaticHandler.class.getName()).log(Level.INFO, "Request by " + request.getRemoteAddr() + "@" + s);
        File f = new File(Webserver.getStaticdir(), s);

        if (f.isDirectory()) {
            File tmp = new File(f.getAbsolutePath(), "index.html");
            if (tmp.exists()) f = tmp;
        }
        String mime = Files.probeContentType(f.toPath());

        PrintWriter out = null;
        try {
            Paths.get(f.getAbsolutePath());
            if (!f.exists()) {
                out = httpServletResponse.getWriter();
                out.write(get404());
                httpServletResponse.setStatus(HttpServletResponse.SC_NOT_FOUND);
            } else if (f.isDirectory() || !f.canRead()) {
                out = httpServletResponse.getWriter();
                out.write(get403());
                httpServletResponse.setStatus(HttpServletResponse.SC_FORBIDDEN);
            } else if (f.isFile()) {
                if (mime.startsWith("text/")) {
                    out = httpServletResponse.getWriter();
                    out.write(getFile(f, true));
                } else {
                    httpServletResponse.getOutputStream().write(getBytes(f));
                }
                httpServletResponse.setStatus(HttpServletResponse.SC_OK);
            }
        } catch (Exception ex) {
            out = httpServletResponse.getWriter();
            out.write(get403());
        }

        switch (FilenameUtils.getExtension(f.getAbsolutePath())) {
            case "eot":
                mime = "application/vnd.ms-fontobject";
                break;

            case "ttf":
                mime = "application/octet-stream";
                break;

            case "svg":
                mime = "image/svg+xml";
                break;

            case "woff":
                mime = "application/font-woff";
                break;

            case "woff2":
                mime = "font/woff2";
                break;
        }

        if (mime == null) mime = "text/html";
        httpServletResponse.setContentType(mime + "; charset=utf-8");
        request.setHandled(true);
    }
}
